BEGIN { push(@INC, ".."); };
use WebminCore;
use Tie::File;

&init_config();
%access = &get_module_acl();

my $instRoot = $gconfig{'hpcmc_root'};

sub svcs_status {
	my $svc = $_[0];
	my @cgpoll = `$instRoot/utilbin/simsh -r service $svc status`;
	return @cgpoll;
}

sub cgconfig_action {
	my $action = $_[0];
	my @cgpoll = `$instRoot/utilbin/simsh -r service cgconfig $action`;
	return @cgpoll;
}

sub cgred_action {
	my $action = $_[0];
	my @cgrpoll = `$instRoot/utilbin/simsh -r service cgred $action`;
	return @cgrpoll;
}

sub parse_cgroups {
	my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
	my %cfileh = ();
	if ( ! -e $cfile ) {
		system("touch $cfile");
		system("chmod 600 $cfile");
	} 
	open(CFILE, $cfile);
	my @cfile = (<CFILE>);
	close(CFILE);
	foreach my $line (@cfile) {
		chomp($line);
		my ($gname,$desc,$subs,$subv,$users,$ausers) = split("\0", $line);
		$cfileh{$gname} = "$desc\0$subs\0$subv\0$users\0$ausers";
	}
	return %cfileh;
}

sub svc_action {
	my $host = $_[0];
	my $svc = $_[1];
	my $action = $_[2];
	my $cmd = "ssh $host service $svc $action";
	my $err = system("ssh $host service $svc $action > /dev/null 2>&1");
	return $err;
}

sub get_cgroupinfo {
	my $cgrp = $_[0];
	my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
        my %cfileh = ();
	tie @cgroups, 'Tie::File', $cfile;
	foreach my $line (@cgroups) {
		if ($line =~ /^$cgrp\0/) {
			my ($gname,$desc,$sub,$subv,$ausers) = split('\0', $line);
			if ($users eq "NULL") {
				$users = "";
			}
			if ($ausers eq "NULL") {
				$ausers = "";
			}
			$subv =~ s/NULL//g;
			$cfileh{'group'} = $gname;
			$cfileh{'desc'} = $desc;
			$cfileh{'subsys'} = $sub;
			$cfileh{'subvals'} = $subv;
			$cfileh{'ausers'} = $ausers;
		}
	}
	untie @cgroups;
	return %cfileh;
}

sub get_cpustats {
	my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
	tie my @cgroups, 'Tie::File', $cfile;
	my $ucpu = 0;
	foreach my $line (@cgroups) {
		my ($gname,$desc,$sub,$subv,$users,$ausers) = split('\0', $line);
		my ($cpu,$mem) = split(",", $subv);
		if ($cpu eq "NULL") {
			next;
		} else {
			$ucpu = $ucpu + $cpu;		
		}
	}
	if ($config{'cm_sup'} == 1) {
		$ucpu = $ucpu + $config{'cm_cpu'};
	}
	my $acpu = 100 - $ucpu;
	untie @cgroups;
	return "$ucpu:$acpu:100";
}

sub get_memstats {
        my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
        tie my @cgroups, 'Tie::File', $cfile;
        my $umem = 0;
        foreach my $line (@cgroups) {
                my ($gname,$desc,$sub,$subv,$users,$ausers) = split('\0', $line);
                my ($cpu,$mem) = split(",", $subv);
                if ($mem eq "NULL") {
			next;
		} else {
			$umem = $umem + $mem;
        	}
	}
	untie @cgroups;
	my @meminfo = `cat /proc/meminfo`;
	my @tmem = grep("/^MemTotal:/", @meminfo);
        my (undef, $fmem) = split(" ", $tmem[0]);
	if ($config{'cm_sup'} == 1) {
		$umem = $umem + $config{'cm_mem'};
	}
	my $amem = $fmem - $umem; 
	return "$umem:$amem:$fmem";
}

sub get_managedusers {
	my $egname;
        if ($_[0] ne "") {
		$egname = $_[0];  
	}
	my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
        my %users = ();
	open(CFILE, $cfile);
        my @cfile = (<CFILE>);
        close(CFILE);
        foreach my $line (@cfile) {
                chomp($line);
                my ($gname,$desc,$subs,$subv,$users,$ausers) = split("\0", $line);
        	if (($egname ne "") and ($egname eq $gname)) {
			next;
		}
		my @musers = split(",", $users);
		foreach my $u (@musers) {
			if ($u ne "NULL") {
				$users{$u}=1;
			}
		}
	}
        return %users;
}

sub add_gent {
	my $gent = $_[0];
	my $error = 0;
	my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
        tie my @cgroups, 'Tie::File', $cfile or $error=1;
	push(@cgroups,$gent);
	untie @cgroups;
	return $error;
}

sub gen_cgconfigf {
	use Fcntl 'O_RDONLY';
	my $error = 0;
	my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
        tie my @cgroups, 'Tie::File', $cfile, mode => O_RDONLY or $error=1;
	tie my @cgfile, 'Tie::File', "/etc/cgconfig.conf" or die $error=2;
	my $cglen = scalar(@cgfile);
	$#cgfile -= $cglen;
	if ($config{'cm_sup'} == 1) {
		push(@cgfile, "mount {\n\tcpu=/cgroup/cpu;\n\tcpuacct=/cgroup/cpuacct;\n\tblkio=/cgroup/blkio;\n}\n");
	} else {
		push(@cgfile, "mount {\n\tcpu=/cgroup/cpu;\n}\n");
	}
	foreach my $line (@cgroups) {
		chomp($line);
		my ($gname,$desc,$sub,$subv,$ausers) = split("\0", $line);
		my ($subcpu,$submem) = split(",",$sub);
		my ($svcpu,$svmem) = split(",",$subv);
		push(@cgfile, "group $gname {\n");
		if ($ausers ne "NULL") {
			push(@cgfile, "\tperm {\n");
			push(@cgfile, "\t\ttask {\n");
			push(@cgfile, "\t\t\tuid=$ausers;\n");
			push(@cgfile, "\t\t\tgid=root;\n");
			push(@cgfile, "\t\t} admin {\n");
			push(@cgfile, "\t\t\tuid=root;\n");
			push(@cgfile, "\t\t\tgid=root;\n");
			push(@cgfile, "\t\t}\n\t}\n");
		}
		if (($subcpu ne "NULL") and ($svcpu ne "NULL")) {
			$svcpu = 1024*($svcpu * .01);
			$svcpu = sprintf "%.0f", $svcpu;
			push(@cgfile, "\tcpu {\n");
			push(@cgfile, "\t\tcpu.shares=$svcpu;\n\t}");
		}
		push(@cgfile, "}\n\n");
	}

	if (($config{'cm_sup'} == 1) and ($config{'cm_mem'} ne "" or $config{'cm_cpu'} ne "")) {
		push(@cgfile, "group cm {\n");
                push(@cgfile, "\tperm {\n");
                push(@cgfile, "\t\ttask {\n");
		if ($config{'cm_tuser'} ne "") {
                	push(@cgfile, "\t\t\tuid=$config{'cm_tuser'};\n");
                } else {
                	push(@cgfile, "\t\t\tuid=root;\n");
		}
		if ($config{'cm_tgroup'} ne "") {
                	push(@cgfile, "\t\t\tgid=$config{'cm_tgroup'};\n");
                } else {
                	push(@cgfile, "\t\t\tgid=root;\n");
		}
                push(@cgfile, "\t\t} admin {\n");
		if ($config{'cm_auser'} ne "") {
                	push(@cgfile, "\t\t\tuid=$config{'cm_auser'};\n");
                } else {
                	push(@cgfile, "\t\t\tuid=root;\n");
		}
		if ($config{'cm_agroup'} ne "") {
                	push(@cgfile, "\t\t\tgid=$config{'cm_agroup'};\n");
                } else {
                	push(@cgfile, "\t\t\tgid=root;\n");
		}
                push(@cgfile, "\t\t}\n\t}\n");
		if ($config{'cm_cpu'} ne "") {
			$cmcpu = 1024*($config{'cm_cpu'} * .01);
			$cmcpu = sprintf "%.0f", $cmcpu;
			push(@cgfile, "\tcpu {\n");
			push(@cgfile, "\t\tcpu.shares=$cmcpu;\n\t}");
		}	
		push(@cgfile, "\tblkio {\n\t}");
		push(@cgfile, "\tcpuacct {\n\t}\n}");
	}	

	untie @cgroups;
	untie @cgfile;
	return $error;
}

sub gen_crconfigf {
	use Fcntl 'O_RDONLY';
        my $error = 0;
        my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
        tie my @cgroups, 'Tie::File', $cfile, mode => O_RDONLY or $error=1;
        tie my @crfile, 'Tie::File', "/etc/cgrules.conf" or die $error=2;
        my $crlen = scalar(@crfile);
        $#crfile -= $crlen;
        foreach my $line (@cgroups) {
                chomp($line);
                my @users; 
		my ($gname,$desc,$sub,$subv,$users,$ausers) = split("\0", $line);
                if ($users eq "NULL") {
			next;
		}
                my ($subcpu,$submem) = split(",",$sub);
		if ($users =~ /,/) {
                        @users  = split(",",$users);
                }
                if (scalar(@users) > 0) {
                        foreach my $line (@users) {
				chomp($line);
				if (($subcpu ne "NULL") and ($submem eq "NULL")) {
                                	push(@crfile, "$line:tkmpinodelib\tcpu\t$gname\n");
                        	} elsif (($subcpu eq "NULL") and ($submem ne "NULL")) {
					push(@crfile, "$line:tkmpinodelib\tmemory\t$gname\n");
				} else {
					push(@crfile, "$line:tkmpinodelib\tcpu,memory\t$gname\n");
				}
			}
                } else {
			chomp($users);
                        if (($subcpu ne "NULL") and ($submem eq "NULL")) {
	                        push(@crfile, "$users:tkmpinodelib\tcpu\t$gname\n");
                        } elsif (($subcpu eq "NULL") and ($submem ne "NULL")) {
                                push(@crfile, "$users:tkmpinodelib\tmemory\t$gname\n");
                        } else {
                                push(@crfile, "$users:tkmpinodelib\tcpu,memory\t$gname\n");
                        }
		}
        }
        return $error;
}

sub prop_cfiles {
	my @files = ("/etc/cgconfig.conf");
	foreach my $line (@files) {
		chomp($line);
		system("$instRoot/utilbin/simcp $line /etc/");
	}
}

sub compare_cgfiles {
	my %results = ();
	@cgfile = `$instRoot/utilbin/simsh -r md5sum /etc/cgconfig.conf`;
	foreach my $line (@cgfile) {
		chomp($line);
		my ($h,$e,$o) = split(":", $line);
		$o =~ s/\s+\/etc\/cgconfig.conf//g; 
		int($e);
		if ($e > 0) {
			if ($ret{'err'} eq "") {
				$results{'err'} = $h;
			} else {
				$results{'err'} = $results{'err'} . ",$h";
			}
		} else {
			$results{$h}=$o;
		}
	} 
	return %results
}

sub compare_crfiles {
	my %results = ();
	@cgfile = `$instRoot/utilbin/simsh -r md5sum /etc/cgrules.conf`;
	foreach my $line (@cgfile) {
		chomp($line);
		my ($h,$e,$o) = split(":", $line);
		$o =~ s/\s+\/etc\/cgrules.conf//g; 
		int($e);
		if ($e > 0) {
			if ($ret{'err'} eq "") {
				$results{'err'} = $h;
			} else {
				$results{'err'} = $results{'err'} . ",$h";
			}
		} else {
			$results{$h}=$o;
		}
	} 
	return %results
}

sub del_grp {
	my $gname = $_[0];
	my $error = 0;
	my $cfile = "$ENV{'WEBMIN_CONFIG'}/cgroups/groups";
        tie my @cgroups, 'Tie::File', $cfile or $error=1;
	my $index = 0;
	foreach my $line (@cgroups) {
		if ($line =~ /$gname\0/) {
			splice(@cgroups, $index, 1);
			last;
		}
		$index++;
	}
	return $error;
}

1;
