BEGIN { push(@INC, ".."); };
use WebminCore;
use File::stat;

&init_config();
%access = &get_module_acl();

my $instRoot = $gconfig{'hpcmc_root'};

sub delete_cubes {
	my $cfile="$instRoot/var/webmin/acubes";
	my @cubedels = @_;
	my @results;

	if (!&test_lock($cfile)) {
		&lock_file($cfile);
	} else {
		print &error($text{'process_locked'});
	} 
	my $nofiles = scalar(@cubedels);
	
	foreach my $line (@cubedels) {
		my ($hc,undef) = split(":", $line);
		push(@hstc, $hc);
	}
	
	my %seen = ();
	my @uniqh = grep { ! $seen{$_} ++ } @hstc;

	my $nohsts = scalar(@uniqh);
	
	print "$text{'process_deleting'} $nofiles $text{'process_files'} $text{'process_across'} $nohsts $text{'process_nodes'}<br><br>";

	foreach my $line (@cubedels) {
		($h,$f) = split(":", $line);
		my $ret = `ssh -o StrictHostKeyChecking=no $h rm -f $f`;
		print "&nbsp&nbsp&nbsp--$text{'process_deleting'} $f $text{'process_on'} $h<br>";
		push(@results, "$h:$?:$f");
	}
	if (&test_lock($cfile)) {
		&unlock_file($cfile);
	}
	my %dfiles = ();
	my $i=1;
	foreach my $line (@results) {
        	my ($h,$r,$f) = split(":", $line);
        	$dfiles{$i} = "$text{'process_node'}:$h,$text{'process_file'}:$f,$text{'process_retcode'}:$r";
        	$i++;
	}
	&webmin_log('Delete', 'Cube', 'Files', \%dfiles);
	return;
}

sub refresh_cubelist {
	my $cloc = $config{'cubes_loc'};
	my $cfile="$instRoot/var/webmin/acubes";
	my $gcmd = $instRoot . '/utilbin/simsh -r "find ' . $cloc . ' -type f -exec stat -c %n:%U:%X {} +"';	
	
	my @files = `$gcmd`;
	foreach my $line (@files) {
		chomp($line);
		$line =~ s/\s+//g;
		my ($snode,$err,$sfile,$susr,$setime) = split(":", $line);
		if ($err > 0) {
			next;
		} else {
			push(@ffiles, "$snode,$sfile,$susr,$setime"); 
		}
	}
	open(ACFILE, ">$cfile");
       	foreach my $line (@ffiles) {
               	print ACFILE "$line\n";
       	}
       	close(ACFILE);
}

sub get_cubes {
	my $cloc = $config{'cubes_loc'};
	my $cfile="$instRoot/var/webmin/acubes";
	my $gcmd = $instRoot . '/utilbin/simsh -r "find ' . $cloc . ' -type f -exec stat -c %n:%U:%X {} +"';	
	my $now =time;
	my @ffiles;	

	if ( $cloc eq "" ) {
        	&ui_print_header(undef, $text{'index_title'}, undef, undef, 1, 1, 0, undef);
        	print &error($text{'index_noloc'});
	}
	
	if ( ! -e "$config{'cubes_loc'}" ) {
        	&ui_print_header(undef, $text{'index_title'}, undef, undef, 1, 1, 0, undef);
        	print &error("\"$config{'cubes_loc'}\" $text{'index_nodir'}");
	}

	if ( -e $cfile ) {
        	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($cfile);
        	if ((($now-$mtime) > 300) or (-z "$cfile")) {
			my @files = `$gcmd`;
			foreach my $line (@files) {
				chomp($line);
				$line =~ s/\s+//g;
				my ($snode,$err,$sfile,$susr,$setime) = split(":", $line);
				if($err > 0) {
					next;
				} else {
					push(@ffiles, "$snode,$sfile,$susr,$setime"); 
				}
			}
			open(ACFILE, ">$cfile");
                	foreach my $line (@ffiles) {
                        	print ACFILE "$line\n";
                	}
                	close(ACFILE);
		} else {
			open(ACFILE, "$cfile");
                	@ffiles = (<ACFILE>);
                	close(ACFILE);

		}
	} else {
		my @files = `$gcmd`;
		foreach my $line (@files) {
			chomp($line);
			$line =~ s/\s+//g;
			my ($snode,$err,$sfile,$susr,$setime) = split(":", $line);
			push(@ffiles, "$snode,$sfile,$susr,$setime"); 
		}
		open(ACFILE, ">$cfile");
                foreach my $line (@ffiles) {
                       	print ACFILE "$line\n";
                }
                close(ACFILE);
	}

	if ( -z "$cfile" ) {
        	&ui_print_header(undef, $text{'index_title'}, undef, undef, 1, 1, 0, undef);
        	print &error("$text{'index_emptyfile'} \"$config{'cubes_loc'}\"");
	}
	return @ffiles;
}

sub date_input
{
	local ($d, $m, $y, $prefix) = @_;
	local $rv;
	$rv .= &ui_textbox($prefix."d", $d, 3)."/";
	$rv .= &ui_select($prefix."m", $m,
                [ map { [ $_, $text{"smonth_".$_} ] } (1..12) ])."/";
	$rv .= &ui_textbox($prefix."y", $y, 5);
	$rv .= &date_chooser_button($prefix."d", $prefix."m", $prefix."y");
	return $rv;
}


1;
