#!/usr/bin/perl
# index.cgi
# Display a table of icons for different types of webmin configuration

use strict;
use warnings;
require './webmin-lib.pl';
our (%in, %text, %gconfig, %config);
my $ver = &get_webmin_version(1);
&ui_print_header(undef, $text{'index_title'}, "", undef, 0, 1, 0,
	undef, undef, undef, &text('index_version', $ver));
my %access = &get_module_acl();
&ReadParse();

my (@wlinks, @wtitles, @wicons);
@wlinks = ( "edit_bind.cgi", "edit_lang.cgi", 
            "edit_log.cgi", "edit_ui.cgi" );
@wtitles = ( $text{'bind_title'}, $text{'lang_title'},
	     $text{'log_title'},  $text{'ui_title'});
@wicons = ( "images/summary.png","images/lang.png",
            "images/log.png","images/ui.png");

# Hide dis-allowed pages
my %allow = map { $_, 1 } split(/\s+/, $access{'allow'});
my %disallow = map { $_, 1 } split(/\s+/, $access{'disallow'});
for(my $i=0; $i<@wlinks; $i++) {
	$wlinks[$i] =~ /edit_(\S+)\.cgi/;
	if (%allow && !$allow{$1} ||
	    $disallow{$1} ||
	    $1 eq "webmin" && $gconfig{'os_type'} eq 'windows') {
		splice(@wlinks, $i, 1);
		splice(@wtitles, $i, 1);
		splice(@wicons, $i, 1);
		$i--;
		}
	}
&icons_table(\@wlinks, \@wtitles, \@wicons);

print &ui_hr();

print &ui_buttons_start();

my %miniserv;
&get_miniserv_config(\%miniserv);

if (&foreign_check("init")) {
	&foreign_require("init");
	my $starting = &init::action_status("webmin");
	print &ui_buttons_row("bootup.cgi",
	      $text{'index_boot'},
	      $text{'index_bootmsg'}.
	      ($miniserv{'inetd'} ? "<b>$text{'index_inetd'}</b>" :
	       !$ENV{'MINISERV_CONFIG'} ? "<b>$text{'index_apache'}</b>" : ""),
	      &ui_hidden("starting", $starting),
	      &ui_radio("boot", $starting == 2 ? 1 : 0,
			[ [ 1, $text{'yes'} ],
			  [ 0, $text{'no'} ] ]));
	}

# Restart Webmin
if (!$miniserv{'inetd'} && $ENV{'MINISERV_CONFIG'}) {
	print &ui_buttons_row("restart.cgi",
		      $text{'index_restart'}, $text{'index_restartmsg'});
	}

# Refresh modules
print &ui_buttons_row("refresh_modules.cgi",
	      $text{'index_refresh'}, $text{'index_refreshmsg'});

print &ui_buttons_end();

if ($in{'refresh'} && defined(&theme_post_change_modules)) {
	# Refresh left menu
	&theme_post_change_modules();
	}

&ui_print_footer("/", $text{'index'});

